#include <memory.h>
#include "ps2matrix3x3.h"

using namespace KazMaths;

const Matrix3x3 Matrix3x3::IDENTITY(1.0f,0.0f,0.0f,
									0.0f,1.0f,0.0f,
									0.0f,0.0f,1.0f);

const Matrix3x3 Matrix3x3::NULLMATRIX(0.0f,0.0f,0.0f,
									  0.0f,0.0f,0.0f,
									  0.0f,0.0f,0.0f);

Matrix3x3::Matrix3x3(void)
{
}

Matrix3x3::~Matrix3x3(void)
{
}

Matrix3x3::Matrix3x3(const Matrix3x3 & rhs)
{
	memcpy(elem, rhs.elem, sizeof(float) * 9);
}

Matrix3x3::Matrix3x3(float _11, float _12, float _13,
					 float _21, float _22, float _23,
					 float _31, float _32, float _33)
{
	elem[0][0] = _11;	elem[0][1] = _12;	elem[0][2] = _13;
	elem[1][0] = _21;	elem[1][1] = _22;	elem[1][2] = _23;
	elem[2][0] = _31;	elem[2][1] = _32;	elem[2][2] = _33;
}

Matrix3x3::Matrix3x3(float _elem[][3])
{
	memcpy(elem, _elem, sizeof(float) * 9);
}

Matrix3x3 operator*( const Matrix3x3 &M1,
                     const Matrix3x3 &M2)
{
    Matrix3x3 ret;

    for(int i = 0; i < 3; i++)
    {
        for(int j = 0; j < 3; j++)
        {
            float Value = 0;
          
            for(int k = 0; k < 3; k++)
            {
                Value += M1(i,k) * M2(k,j);
            }

            ret(i,j) = Value;
        }
    }

    return ret;
}

Vector3 operator * ( const Matrix3x3 &M,
					 const Vector3 &V )
{
    Vector3 ret;

	ret.x = M(0,0) * V.x + M(0,1) * V.y + M(0,2) * V.z;
	ret.y = M(1,0) * V.x + M(1,1) * V.y + M(1,2) * V.z;
	ret.z = M(2,0) * V.x + M(2,1) * V.y + M(2,2) * V.z;

    return ret;
}

Matrix3x3 Matrix3x3::GetInverse() const
{
	Matrix3x3 inv;
	inv.elem[0][0] = elem[1][1]*elem[2][2] -
					 elem[1][2]*elem[2][1];
    inv.elem[0][1] = elem[0][2]*elem[2][1] -
					 elem[0][1]*elem[2][2];
    inv.elem[0][2] = elem[0][1]*elem[1][2] -
					 elem[0][2]*elem[1][1];
    inv.elem[1][0] = elem[1][2]*elem[2][0] -
					 elem[1][0]*elem[2][2];
    inv.elem[1][1] = elem[0][0]*elem[2][2] -
					 elem[0][2]*elem[2][0];
    inv.elem[1][2] = elem[0][2]*elem[1][0] -
					 elem[0][0]*elem[1][2];
    inv.elem[2][0] = elem[1][0]*elem[2][1] -
				     elem[1][1]*elem[2][0];
    inv.elem[2][1] = elem[0][1]*elem[2][0] -
					 elem[0][0]*elem[2][1];
    inv.elem[2][2] = elem[0][0]*elem[1][1] -
					 elem[0][1]*elem[1][0];

    float fDet =
        elem[0][0]*inv.elem[0][0] +
        elem[0][1]*inv.elem[1][0]+
        elem[0][2]*inv.elem[2][0];

    if ( Abs(fDet) <= 0.0000001f )
		return Matrix3x3::NULLMATRIX;

    float fInvDet = 1.0f/fDet;
    for (int i = 0; i < 3; i++)
    {
        for (int j = 0; j < 3; j++)
			inv.elem[i][j] *= fInvDet;
    }

	return inv;
}

void Matrix3x3::Invert()
{
	Matrix3x3 inv;
	inv.elem[0][0] = elem[1][1]*elem[2][2] -
					 elem[1][2]*elem[2][1];
    inv.elem[0][1] = elem[0][2]*elem[2][1] -
					 elem[0][1]*elem[2][2];
    inv.elem[0][2] = elem[0][1]*elem[1][2] -
					 elem[0][2]*elem[1][1];
    inv.elem[1][0] = elem[1][2]*elem[2][0] -
					 elem[1][0]*elem[2][2];
    inv.elem[1][1] = elem[0][0]*elem[2][2] -
					 elem[0][2]*elem[2][0];
    inv.elem[1][2] = elem[0][2]*elem[1][0] -
					 elem[0][0]*elem[1][2];
    inv.elem[2][0] = elem[1][0]*elem[2][1] -
				     elem[1][1]*elem[2][0];
    inv.elem[2][1] = elem[0][1]*elem[2][0] -
					 elem[0][0]*elem[2][1];
    inv.elem[2][2] = elem[0][0]*elem[1][1] -
					 elem[0][1]*elem[1][0];

    float fDet =
        elem[0][0]*inv.elem[0][0]+
        elem[0][1]*inv.elem[1][0]+
        elem[0][2]*inv.elem[2][0];

    if ( Abs(fDet) <= 0.0000001f )
		return;

    float fInvDet = 1.0f/fDet;
    for (int i = 0; i < 3; i++)
    {
        for (int j = 0; j < 3; j++)
			elem[i][j] = inv.elem[i][j] * fInvDet;
    }
}

void Orthonormalise( KazMaths::Matrix3x3 & M )
{
	Vector3 X(M(0,0),M(1,0),M(2,0));
    Vector3 Y(M(0,1),M(1,1),M(2,1));
    Vector3 Z;

    X.Normalise();
    Z = CrossProduct(X,Y).Normalise();
    Y = CrossProduct(Z,X).Normalise();

    M(0,0) = X.x; M(0,1) = Y.x; M(0,2) = Z.x;
    M(1,0) = X.y; M(1,1) = Y.y; M(1,2) = Z.y;
    M(2,0) = X.z; M(2,1) = Y.z; M(2,2) = Z.z;
}
